//===========================================================================
//  @(#) $Name: arts++-1-1-a13 $
//  @(#) $Id: ArtsNetMatrixAggregator.hh,v 1.2 2004/04/21 23:51:27 kkeys Exp $
//===========================================================================
//  Copyright Notice
//
//  By accessing this software, arts++, you are duly informed
//  of and agree to be bound by the conditions described below in this
//  notice:
//
//  This software product, arts++, is developed by Daniel W. McRobb, and
//  copyrighted(C) 1998 by the University of California, San Diego
//  (UCSD), with all rights reserved.  UCSD administers the CAIDA grant,
//  NCR-9711092, under which part of this code was developed.
//
//  There is no charge for arts++ software.  You can redistribute it
//  and/or modify it under the terms of the GNU Lesser General Public
//  License, Version 2.1, February 1999, which is incorporated by
//  reference herein.
//
//  arts++ is distributed WITHOUT ANY WARRANTY, IMPLIED OR EXPRESS, OF
//  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that the use
//  of it will not infringe on any third party's intellectual
//  property rights.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with arts++.  Copies can also be obtained from:
//
//    http://www.gnu.org/copyleft/lesser.html
//
//  or by writing to:
//
//  Free Software Foundation, Inc.
//  59 Temple Place, Suite 330
//  Boston, MA 02111-1307
//  USA
//
//  Or contact:
//
//    info@caida.org
//===========================================================================

#ifndef _ARTSNETMATRIXAGGREGATOR_HH_
#define _ARTSNETMATRIXAGGREGATOR_HH_

#include <map>
#include <vector>

#include "ArtsNetMatrix.hh"

//----------------------------------------------------------------------------
//     class ArtsNetMatrixKeyValue 
//----------------------------------------------------------------------------
//!  This class is used as a key in a map of counters used by the
//!  ArtsNetMatrixAggregator class.
//----------------------------------------------------------------------------
class ArtsNetMatrixKeyValue
{
public:
  //--------------------------------------------------------------------------
  //       bool operator < (const ArtsNetMatrixKeyValue keyValue) const 
  //..........................................................................
  //!  less-than comparison.  This is used by the ArtsNetMatrixAggregator
  //!  class for comparison of keys in an STL map of counters. In other
  //!  words, it determines ordering of elements in the map.
  //--------------------------------------------------------------------------
  bool operator < (const ArtsNetMatrixKeyValue keyValue) const;
  
  //--------------------------------------------------------------------------
  //       bool operator == (const ArtsNetMatrixKeyValue keyValue) const 
  //..........................................................................
  //!  Returns true if *this is equal to keyValue.
  //--------------------------------------------------------------------------
  bool operator == (const ArtsNetMatrixKeyValue keyValue) const;
  ipv4addr_t  srcNet;
  uint8_t     srcMaskLen;
  ipv4addr_t  dstNet;
  uint8_t     dstMaskLen;
};


//---------------------------------------------------------------------------
//  class ArtsNetMatrixAggregator
//---------------------------------------------------------------------------
//!  This class is used to aggregate net matrix objects in the time
//!  domain.
//---------------------------------------------------------------------------
class ArtsNetMatrixAggregator
{
public:
  //--------------------------------------------------------------------------
  //!  Holds a packet and byte counter.
  //--------------------------------------------------------------------------
  typedef struct {
    uint64_t  Pkts;
    uint64_t  Bytes;
  } counter_t;

  //-------------------------------------------------------------------------
  //               ArtsNetMatrixAggregator(const Arts & arts)
  //.........................................................................
  //!  construct from an Arts object.
  //-------------------------------------------------------------------------
  ArtsNetMatrixAggregator(const Arts & arts);

  //--------------------------------------------------------------------------
  //                       ~ArtsNetMatrixAggregator()
  //..........................................................................
  //!  destructor
  //--------------------------------------------------------------------------
  ~ArtsNetMatrixAggregator();

  //-------------------------------------------------------------------------
  //                       void Add(const Arts & arts)
  //.........................................................................
  //!  Add the contents of an Arts object.  The input Arts object must be
  //!  a net matrix object.  The effect here is to add any new net matrix
  //!  entries and add to the counters for existing net matrix entries.
  //-------------------------------------------------------------------------
  void Add(const Arts & arts);
  
  //-------------------------------------------------------------------------
  //              ArtsNetMatrix *ConvertToArtsNetMatrix() const
  //.........................................................................
  //!  Return an ArtsNetMatrix object (dynamically allocated) containing
  //!  the aggregate data.  The caller is responsible for calling delete()
  //!  on the returned ArtsNetMatrix pointer.
  //-------------------------------------------------------------------------
  ArtsNetMatrix *ConvertToArtsNetMatrix() const;
  
private:
  ArtsHeader                               _header;
  std::vector<ArtsAttribute>                    _attributes;
 std::map<ArtsNetMatrixKeyValue,counter_t,std::less<ArtsNetMatrixKeyValue> >  _netCounters;
  
};

#endif  // _ARTSNETMATRIXAGGREGATOR_HH_
